@@ -71,6 +71,7 @@ def evaluation(
     if isinstance(model, tf.keras.models.Model):
         input_shape = model.input_shape[1:-1] if input_shape is None else input_shape[:2]
         model_interf = change_model_input_shape(model, input_shape)
+        print(">>>> Using input_shape {} for Keras model.".format(input_shape))
     elif isinstance(model, TFLiteModelInterf) or (isinstance(model, str) and model.endswith(".tflite")):
         model_interf = model if isinstance(model, TFLiteModelInterf) else TFLiteModelInterf(model)
         input_shape = model_interf.input_shape
@@ -136,11 +137,6 @@ def parse_timm_log(log_file, pick_keys=None):
     if val_acc[-1] > 1:
         val_acc = [ii / 100.0 for ii in val_acc]
 
-    # train_loss = [float(ii.split('Loss:')[1].strip().split(" ")[1][1:-1]) for ii in aa if train_epoch_end_pattern in ii]
-    # lr = [float(ii.split('LR:')[1].strip().split(" ")[0]) for ii in aa if train_epoch_end_pattern in ii]
-    # val_loss = [float(ii.split('Loss:')[1].strip().split(" ")[1][1:-1]) for ii in aa if test_epoch_end_pattern in ii]
-    # val_acc = [float(ii.split('Acc@1:')[1].strip().split("Acc@5:")[0].split("(")[1].split(")")[0]) for ii in aa if test_epoch_end_pattern in ii]
-
     # print(f"{len(train_loss) = }, {len(lr) = }, {len(val_loss) = }, {len(val_acc) = }")
     hh = {"loss": train_loss, "lr": lr, "val_loss": val_loss, "val_acc": val_acc}
     return hh if pick_keys is None else {kk: hh[kk] for kk in pick_keys}
@@ -162,7 +158,7 @@ def combine_hist_into_one(hist_list, save_file=None):
     return hh
 
 
-def plot_and_peak_scatter(ax, array, peak_method, label, skip_first=0, color=None, va="bottom", **kwargs):
+def plot_and_peak_scatter(ax, array, peak_method, label, skip_first=0, color=None, va="bottom", fontsize=12, **kwargs):
     array = array[skip_first:]
     for id, ii in enumerate(array):
         if tf.math.is_nan(ii):
@@ -172,7 +168,7 @@ def plot_and_peak_scatter(ax, array, peak_method, label, skip_first=0, color=Non
     pp = peak_method(array)
     vv = array[pp]
     ax.scatter(pp + skip_first, vv, color=color, marker="v")
-    ax.text(pp + skip_first, vv, "{:.4f}".format(vv), va=va, ha="right", color=color, fontsize=9, rotation=0)
+    ax.text(pp + skip_first, vv, "{:.4f}".format(vv), va=va, ha="right", color=color, fontsize=fontsize, rotation=0)
 
 
 def plot_hists(hists, names=None, base_size=6, addition_plots=["lr"], text_va=["bottom"], skip_first=0):
@@ -185,6 +181,7 @@ def plot_hists(hists, names=None, base_size=6, addition_plots=["lr"], text_va=["
     fig, axes = plt.subplots(1, num_axes, figsize=(num_axes * base_size, base_size))
     hists = [hists] if isinstance(hists, (str, dict)) else hists
     names = names if isinstance(names, (list, tuple)) else [names]
+    fontsize = base_size * 2 - 2
     for id, hist in enumerate(hists):
         name = names[min(id, len(names) - 1)] if names != None else None
         cur_va = text_va[id % len(text_va)]
@@ -194,21 +191,21 @@ def plot_hists(hists, names=None, base_size=6, addition_plots=["lr"], text_va=["
                 hist = json.load(ff)
         name = name if name != None else str(id)
 
-        plot_and_peak_scatter(axes[0], hist["loss"], np.argmin, name + " loss", skip_first, color=None, va=cur_va)
+        plot_and_peak_scatter(axes[0], hist["loss"], np.argmin, name + " loss", skip_first, color=None, va=cur_va, fontsize=fontsize)
         color = axes[0].lines[-1].get_color()
         val_loss = hist.get("val_loss", [])
         if len(val_loss) > 0 and "val_loss" not in addition_plots:
-            plot_and_peak_scatter(axes[0], val_loss, np.argmin, name + " val_loss", skip_first, color, va=cur_va, linestyle="--")
+            plot_and_peak_scatter(axes[0], val_loss, np.argmin, name + " val_loss", skip_first, color, cur_va, fontsize, linestyle="--")
         acc = hist.get("acc", hist.get("accuracy", []))
         if len(acc) > 0:  # For timm log
-            plot_and_peak_scatter(axes[1], acc, np.argmax, name + " accuracy", skip_first, color=color, va=cur_va)
+            plot_and_peak_scatter(axes[1], acc, np.argmax, name + " accuracy", skip_first, color=color, va=cur_va, fontsize=fontsize)
         val_acc = hist.get("val_acc", hist.get("val_accuracy", []))
-        plot_and_peak_scatter(axes[1], val_acc, np.argmax, name + " val_accuracy", skip_first, color=color, va=cur_va, linestyle="--")
+        plot_and_peak_scatter(axes[1], val_acc, np.argmax, name + " val_accuracy", skip_first, color, cur_va, fontsize, linestyle="--")
         if addition_plots is not None and len(addition_plots) != 0:
             for id, ii in enumerate(addition_plots):
                 if len(hist.get(ii, [])) > 0:
                     peak_method = np.argmin if "loss" in ii else np.argmax
-                    plot_and_peak_scatter(axes[2 + id], hist[ii], peak_method, name + " " + ii, skip_first, color=color, va=cur_va)
+                    plot_and_peak_scatter(axes[2 + id], hist[ii], peak_method, name + " " + ii, skip_first, color, cur_va, fontsize=fontsize)
     for ax in axes:
         ax.legend()
         ax.grid(True)
